@@ -4,57 +4,96 @@ package Moose::Object;
 use strict;
 use warnings;
 
-use Moose::Meta::Class;
-use metaclass 'Moose::Meta::Class';
+use if ( not our $__mx_is_compiled ), 'Moose::Meta::Class';
+use if ( not our $__mx_is_compiled ), metaclass => 'Moose::Meta::Class';
 
 use Carp 'confess';
 
-our $VERSION = '0.07';
+our $VERSION   = '0.54';
+our $AUTHORITY = 'cpan:STEVAN';
 
 sub new {
     my $class = shift;
-    my %params;
+    my $params = $class->BUILDARGS(@_);
+    my $self = $class->meta->new_object(%$params);
+    $self->BUILDALL($params);
+    return $self;
+}
+
+sub BUILDARGS {
+    my $class = shift;
     if (scalar @_ == 1) {
-        (ref($_[0]) eq 'HASH')
-            || confess "Single parameters to new() must be a HASH ref";
-        %params = %{$_[0]};
-    }
+        if (defined $_[0]) {
+            (ref($_[0]) eq 'HASH')
+                || confess "Single parameters to new() must be a HASH ref";
+            return {%{$_[0]}};
+        } 
+        else {
+            return {}; # FIXME this is compat behavior, but is it correct?
+        }
+    } 
     else {
-        %params = @_;
+        return {@_};
     }
-	my $self = $class->meta->new_object(%params);
-	$self->BUILDALL(\%params);
-	return $self;
 }
 
 sub BUILDALL {
-	return unless $_[0]->can('BUILD');	    
-	my ($self, $params) = @_;
-	foreach my $method (reverse $self->meta->find_all_methods_by_name('BUILD')) {
-		$method->{code}->($self, $params);
-	}
+    # NOTE: we ask Perl if we even 
+    # need to do this first, to avoid
+    # extra meta level calls
+    return unless $_[0]->can('BUILD');    
+    my ($self, $params) = @_;
+    foreach my $method (reverse $self->meta->find_all_methods_by_name('BUILD')) {
+        $method->{code}->body->($self, $params);
+    }
 }
 
 sub DEMOLISHALL {
-	return unless $_[0]->can('DEMOLISH');	    
-	my $self = shift;
-	foreach my $method ($self->meta->find_all_methods_by_name('DEMOLISH')) {
-		$method->{code}->($self);
-	}	
+    my $self = shift;    
+    foreach my $method ($self->meta->find_all_methods_by_name('DEMOLISH')) {
+        $method->{code}->body->($self);
+    }
 }
 
-sub DESTROY { goto &DEMOLISHALL }
+sub DESTROY { 
+    # NOTE: we ask Perl if we even 
+    # need to do this first, to avoid
+    # extra meta level calls    
+    return unless $_[0]->can('DEMOLISH');
+    # if we have an exception here ...
+    if ($@) {
+        # localize the $@ ...
+        local $@;
+        # run DEMOLISHALL ourselves, ...
+        $_[0]->DEMOLISHALL;
+        # and return ...
+        return;
+    }
+    # otherwise it is normal destruction
+    $_[0]->DEMOLISHALL;
+}
+
+# support for UNIVERSAL::DOES ...
+BEGIN {
+    my $does = UNIVERSAL->can("DOES") ? "SUPER::DOES" : "isa";
+    eval 'sub DOES {
+        my ( $self, $class_or_role_name ) = @_;
+        return $self->'.$does.'($class_or_role_name)
+            || $self->does($class_or_role_name);
+    }';
+}
 
 # new does() methods will be created 
 # as approiate see Moose::Meta::Role
 sub does {
     my ($self, $role_name) = @_;
     (defined $role_name)
-        || confess "You much supply a role name to does()";
+        || confess "You must supply a role name to does()";
     my $meta = $self->meta;
     foreach my $class ($meta->class_precedence_list) {
+        my $m = $meta->initialize($class);
         return 1 
-            if $meta->initialize($class)->does_role($role_name);            
+            if $m->can('does_role') && $m->does_role($role_name);            
     }
     return 0;   
 }
@@ -71,7 +110,7 @@ sub does {
 sub dump { 
     my $self = shift;
     require Data::Dumper;
-    $Data::Dumper::Maxdepth = shift if @_;
+    local $Data::Dumper::Maxdepth = shift if @_;
     Data::Dumper::Dumper $self;
 }
 
@@ -106,7 +145,12 @@ This will return the metaclass associated with the given class.
 
 =item B<new>
 
-This will create a new instance and call C<BUILDALL>.
+This will call C<BUILDARGS>, create a new instance and call C<BUILDALL>.
+
+=item B<BUILDARGS>
+
+This method processes an argument list into a hash reference. It is used by
+C<new>.
 
 =item B<BUILDALL>
 
@@ -122,6 +166,12 @@ This will call every C<DEMOLISH> method in the inheritance hierarchy.
 This will check if the invocant's class C<does> a given C<$role_name>. 
 This is similar to C<isa> for object, but it checks the roles instead.
 
+=item B<DOES ($class_or_role_name)>
+
+A Moose Role aware implementation of L<UNIVERSAL/DOES>.
+
+C<DOES> is equivalent to C<isa> or C<does>.
+
 =item B<dump ($maxdepth)>
 
 Cmon, how many times have you written the following code while debugging:
@@ -145,11 +195,11 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006 by Infinity Interactive, Inc.
+Copyright 2006-2008 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
 
-=cut
\ No newline at end of file
+=cut
